@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import argparse
+import contextlib
 import itertools
 import os
 import re
@@ -8,6 +9,7 @@ import shlex
 import signal
 import subprocess
 import sys
+from functools import partial
 from pathlib import Path
 from typing import TYPE_CHECKING, Mapping, NamedTuple, Sequence, cast
 
@@ -24,7 +26,7 @@ from pdm.utils import deprecation_warning, expand_env_vars, is_path_relative_to
 
 if TYPE_CHECKING:
     from types import FrameType
-    from typing import Any, Callable, Iterator, TypedDict
+    from typing import Any, Iterator, TypedDict
 
     from pdm.environments import BaseEnvironment
     from pdm.project import Project
@@ -163,24 +165,24 @@ class TaskRunner:
             return self.project.environment
         tool_config = metadata.pop("tool", {})
         script_project = self.project.core.create_project()
-        script_project.pyproject.set_data({"project": metadata, "tool": tool_config})
-        md5_kwargs = {}
-        # for python >= 3.9, indicate that md5 is not used in a security context
-        if sys.version_info >= (3, 9):
-            md5_kwargs = {"usedforsecurity": False}
-        venv_name = hashlib.md5(os.path.realpath(script_file).encode("utf-8"), **md5_kwargs).hexdigest()
+        script_project.pyproject.set_data(
+            {"project": {"name": "temp-project", **metadata, "version": "0.0.0"}, "tool": tool_config}
+        )
+        venv_name = hashlib.md5(os.path.realpath(script_file).encode("utf-8"), usedforsecurity=False).hexdigest()
         venv_backend = BACKENDS[script_project.config["venv.backend"]](script_project, None)
         venv = venv_backend.get_location(None, venv_name)
-        if venv.exists() and not self.recreate_env:
-            self.project.core.ui.info(f"Reusing existing script environment: {venv}", verbosity=termui.Verbosity.DETAIL)
-        else:
-            self.project.core.ui.info(f"Creating environment for script: {venv}", verbosity=termui.Verbosity.DETAIL)
-            venv = venv_backend.create(venv_name=venv_name, force=True)
-        env = PythonEnvironment(script_project, python=get_venv_python(venv).as_posix())
-        script_project._python = env.interpreter
-        env.project = script_project  # keep a strong reference to the project
-        if reqs := script_project.get_dependencies():
-            install_requirements(reqs, env, clean=True)
+        with contextlib.ExitStack() as stack:
+            if venv.exists() and not self.recreate_env:
+                stack.enter_context(self.project.core.ui.open_spinner("[info]Reusing existing script environment"))
+            else:
+                stack.enter_context(self.project.core.ui.open_spinner("[info]Creating environment for script"))
+                venv = venv_backend.create(venv_name=venv_name, force=True)
+            self.project.core.ui.info(f"Script environment location: {venv}", verbosity=termui.Verbosity.DETAIL)
+            env = PythonEnvironment(script_project, python=get_venv_python(venv).as_posix())
+            script_project._python = env.interpreter
+            env.project = script_project  # keep a strong reference to the project
+            if reqs := script_project.get_dependencies():
+                install_requirements(reqs, env, clean=True)
         return env
 
     def get_task(self, script_name: str) -> Task | None:
@@ -290,9 +292,11 @@ class TaskRunner:
                 signum = signal.SIGTERM
             process.send_signal(signum)
 
+        process_env = os.environ.copy()
+        process_env.update({"PDM_RUN_CWD": str(Path.cwd())})
         handle_term = signal.signal(signal.SIGTERM, forward_signal)
         handle_int = signal.signal(signal.SIGINT, forward_signal)
-        process = subprocess.Popen(process_cmd, cwd=cwd, shell=shell, bufsize=0, close_fds=False)
+        process = subprocess.Popen(process_cmd, cwd=cwd, shell=shell, bufsize=0, close_fds=False, env=process_env)
         retcode = process.wait()
         signal.signal(signal.SIGTERM, handle_term)
         signal.signal(signal.SIGINT, handle_int)
@@ -342,11 +346,8 @@ class TaskRunner:
         elif kind == "composite":
             assert isinstance(value, list)
 
-        self.project.core.ui.echo(
-            f"Running {task}: [success]{args}[/]",
-            err=True,
-            verbosity=termui.Verbosity.DETAIL,
-        )
+        self.display_task(task, args)
+
         if kind == "composite":
             args = list(args)
             should_interpolate = any(RE_ARGS_PLACEHOLDER.search(script) for script in value)
@@ -367,6 +368,19 @@ class TaskRunner:
             return composite_code
         return self._run_process(args, chdir=True, shell=shell, **merge_options(self.global_options, options, opts))  # type: ignore[misc]
 
+    def display_task(self, task: Task, args: Sequence[str]) -> None:
+        """Display a task given current verbosity and settings"""
+        is_verbose = self.project.core.ui.verbosity >= termui.Verbosity.DETAIL
+        if not (is_verbose or self.project.config.get("scripts.show_header")):
+            return
+        args = task.args if task.kind == "composite" else args
+        content = args if is_verbose else task.short_description
+        self.project.core.ui.echo(
+            f"Running {task}: [success]{content}[/]",
+            err=True,
+            verbosity=termui.Verbosity.NORMAL,
+        )
+
     def run(
         self,
         command: str,
@@ -514,20 +528,21 @@ class Command(BaseCommand):
         sys.exit(exit_code)
 
 
-def run_script_if_present(script_name: str) -> Callable:
-    """Helper to create a signal handler to run specific script"""
+def run_script_if_present(script_name: str, sender: Project, hooks: HookManager, **kwargs: Any) -> None:
+    """A signal handler to run a script if present in the project."""
 
-    def handler(sender: Project, hooks: HookManager, **kwargs: Any) -> None:
-        runner = TaskRunner(sender, hooks)
-        task = runner.get_task(script_name)
-        if task is None:
-            return
-        exit_code = runner.run_task(task)
-        if exit_code != 0:
-            sys.exit(exit_code)
-
-    return handler
+    runner = TaskRunner(sender, hooks)
+    task = runner.get_task(script_name)
+    if task is None:
+        return
+    exit_code = runner.run_task(task)
+    if exit_code != 0:
+        sys.exit(exit_code)
+    # reload project files in case the script modified them
+    sender.project_config.reload()
+    sender.pyproject.reload()
+    sender.lockfile.reload()
 
 
 for hook in pdm_signals:
-    pdm_signals.signal(hook).connect(run_script_if_present(hook), weak=False)
+    pdm_signals.signal(hook).connect(partial(run_script_if_present, hook), weak=False)
