@@ -24,7 +24,6 @@ from pdm.models.markers import Marker, get_marker
 from pdm.models.setup import Setup
 from pdm.models.specifiers import PySpecSet, fix_legacy_specifier, get_specifier
 from pdm.utils import (
-    PACKAGING_22,
     add_ssh_scheme_to_git_uri,
     comparable_version,
     normalize_name,
@@ -239,6 +238,10 @@ class NamedRequirement(Requirement):
         return f"{self.project_name}{extras}{self.specifier or ''}{self._format_marker()}"
 
 
+# Cache for checked paths to avoid checking the same path multiple times
+_checked_paths: set[Path] = set()
+
+
 @dataclasses.dataclass(eq=False)
 class FileRequirement(Requirement):
     url: str = ""
@@ -323,12 +326,17 @@ class FileRequirement(Requirement):
             else:
                 self.path = Path(relpath)
 
-        if self.path is not None:
+        if self.path is not None and self.absolute_path not in _checked_paths:
             # For relative path, we don't resolve URL now, so the path may still contain fragments,
             # it will be handled in `relocate()` method.
-            result = Setup.from_directory(self.absolute_path)  # type: ignore[arg-type]
+            assert self.absolute_path is not None
+            # There is a risk of infinite recursion here if the project file contains file dependencies
+            # referring to itself, mark the path as being checked to avoid that.
+            _checked_paths.add(self.absolute_path)
+            result = Setup.from_directory(self.absolute_path)
             if result.name:
                 self.name = result.name
+            _checked_paths.discard(self.absolute_path)
 
         if self.url:
             self._parse_name_from_url()
@@ -480,8 +488,6 @@ def parse_as_pkg_requirement(line: str) -> PackageRequirement:
     try:
         return PackageRequirement(line)
     except InvalidRequirement:
-        if not PACKAGING_22:  # We can't do anything, reraise the error.
-            raise
         new_line = fix_legacy_specifier(line)
         return PackageRequirement(new_line)
 
